import { selectSource } from './common';

interface BuilderMemory extends CreepMemory {
  building: boolean | undefined;
}

export function runAsBuilder(creep: Creep) {
  const memory = creep.memory as BuilderMemory;

  if (memory.building && creep.store[RESOURCE_ENERGY] === 0) {
    memory.building = false;
  }

  if (!memory.building && creep.store.getFreeCapacity() === 0) {
    memory.building = true;
  }

  if (!memory.sourceId) {
    memory.sourceId = selectSource(creep.room);
  }

  if (memory.building) {
    if (build(creep)) {
      return;
    }
    if (repair(creep)) {
      return;
    }
    const spawns = creep.room.find(FIND_MY_SPAWNS);
    if (spawns.length > 0) {
      creep.say('To spawn');
      creep.moveTo(spawns[0]);
    }
  } else {
    const source = Game.getObjectById(memory.sourceId);
    if (source && creep.harvest(source) == ERR_NOT_IN_RANGE) {
      creep.say('🔄 harvest');
      creep.moveTo(source, { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  }
}

function build(creep: Creep): boolean {
  const targets = creep.room.find(FIND_MY_CONSTRUCTION_SITES);
  if (targets.length === 0) {
    return false;
  }
  if (creep.build(targets[0]) == ERR_NOT_IN_RANGE) {
    creep.say('🚧 build');
    creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
  }
  return true;
}

function repair(creep: Creep) {
  const target = findRepairTarget(creep);
  if (!target) {
    return false;
  }
  if (creep.repair(target) == ERR_NOT_IN_RANGE) {
    creep.say('🚧 repair');
    creep.moveTo(target, { visualizePathStyle: { stroke: '#ffffff' } });
  }
  return true;
}

function findRepairTarget(creep: Creep): Structure | undefined {
  const nearest = creep.pos.findInRange(FIND_STRUCTURES, 3, {
    filter: (t) => t.hits < t.hitsMax && t.structureType !== STRUCTURE_ROAD
  });
  if (nearest.length > 0) {
    nearest.sort((t1, t2) => t1.hits - t2.hits);
    return nearest[0];
  }
  const targets = creep.room.find(FIND_STRUCTURES, {
    filter: (t) => t.hits < t.hitsMax && t.structureType !== STRUCTURE_ROAD
  });
  if (targets.length > 0) {
    targets.sort((t1, t2) => t1.hits - t2.hits);
    return targets[0];
  }
  return undefined;
}
